# -*- coding: utf-8 -*-
"""
Spyder Editor


import pandas
import numpy
import matplotlib.pyplot as plt
import scipy.optimize as opt

def sigmoid_log(x, growth_rate, midpoint, intercept):
    return 2 / (1. + numpy.exp(-growth_rate * (x - midpoint))) + intercept

def sigmoid(x, growth_rate, midpoint, intercept):
    return 100 / (1. + numpy.exp(-growth_rate * (x - midpoint))) + intercept


# Read data from csv
data = pandas.read_csv("sweetness16_pp14.csv")
sweeteners = list(numpy.unique(data["type_substance"]))

# Do loglog fit on all sweeteners
print("loglog linear fit")
fig1, ax = plt.subplots(len(sweeteners), sharex="all")
for n,type_subst in enumerate(sweeteners):
  x_data = data[data["type_substance"]==type_subst]["log_concentration_"]
  y_data = data[data["type_substance"]==type_subst]["log_rating_"]
  P = numpy.polyfit(x_data, y_data, 1)
  ss_res = sum((y_data-numpy.polyval(P, x_data))**2)
  ss_tot = sum((y_data-numpy.mean(y_data))**2)
  Rsq = 1-ss_res/ss_tot
  ax[n].plot( x_data, y_data, label=f"{type_subst} data", linestyle="none", marker="o", color="C0" )
  x = numpy.linspace(min(x_data), max(y_data), 100) 
  y = numpy.polyval(P, x)
  ax[n].plot(x, y, 
             label=f"{type_subst} fit (R²={Rsq:1.4f})",
             color="C0")
  ax[n].set_ylabel("log rating")
  ax[n].legend()
  print(f"{type_subst}: {P[0]:2.2f}x + {P[1]:2.2f} (R²={Rsq:1.4f})")
ax[-1].set_xlabel("log concentration")
fig1.suptitle("loglog linear")
fig1.show()

# Do sigmoid polyfit on all sweeteners
print("loglog Sigmoid fit")
fig2, ax = plt.subplots(len(sweeteners), sharex="all")
for n,type_subst in enumerate(sweeteners):
  x_data = numpy.array(data[data["type_substance"]==type_subst]["log_concentration_"])
  y_data = numpy.array(data[data["type_substance"]==type_subst]["log_rating_"])
  (growth_rate, midpoint, intercept), pcov = opt.curve_fit(sigmoid_log, x_data, y_data)
  ss_res = sum((y_data-sigmoid_log(x_data, growth_rate, midpoint, intercept))**2)
  ss_tot = sum((y_data-numpy.mean(y_data))**2)
  Rsq = 1-ss_res/ss_tot
  x = numpy.linspace(min(x_data), max(y_data), 100) 
  y = sigmoid_log(x, growth_rate, midpoint, intercept)
  ax[n].plot( x_data, y_data,
              label=f"{type_subst} data", linestyle="none", marker="o", color="C0" )
  ax[n].plot(x, y, label=f"{type_subst} fit (R²={Rsq:1.4f})", color="C0")
  ax[n].set_ylabel("log rating")
  
  ax[n].legend()
  print(f"{type_subst}: 2/(1+exp({-growth_rate:2.2f} * (x - {midpoint:2.2f}))) + {intercept:2.2f} (R²={Rsq:1.4f})")
ax[-1].set_xlabel("log concentration")
fig2.suptitle("loglog sigmoid")
fig2.show()


  