%% ery_4a_secondlevel_m14_s1_a2_set_default_options.m
%
% USAGE
% Set options used in various core secondlevel scripts included in
% CANlab_help_examples (LaBGAS fork)
% 
% LaBGAS notes
% 
% - Always make a study-specific copy of this script in your code subdataset, do NOT edit in the repo!
% - The below can be considered LaBGAS default options, but decisions will be study-specific!
% - Various options added by @lukasvo76 spring 21 and May 22
%
%__________________________________________________________________________
%
% modified by: Lukas Van Oudenhove
% date:   Dartmouth, May, 2022
%
%__________________________________________________________________________
% @(#)% a2_set_default_options.m         v2.0
% last modified: 2022/05/28


%% PREP_2_LOAD_IMAGE_DATA_AND_SAVE & PREP_3_CALC_UNIVARIATE_CONTRAST_MAPS_AND_SAVE
% --------------------------------------------------------------------------------

dofullplot = true;         % default true  Can set to false to save time
omit_histograms = false;     % default false Histograms not useful for large samples 
dozipimages = false;        % default false to avoid load on data upload/download when re-running often, true is useful to save space when running final analyses; lukasvo76: changed from original CANlab default


%% PREP_3_RUN_SECOND_LEVEL_REGRESSION_AND_SAVE
% --------------------------------------------

dorobust = false;            % robust statistics for voxel-based GLM [true, false] -- default true
myscaling_glm = 'raw';      % 'raw', 'scaled', or 'scaledcontrasts'; 
                            % 'scaled': use z-scored condition images prior to computing contrasts
                            % 'scaled_contrasts': l2norm contrasts after computing them
design_matrix_type = 'onesample';   % 'group', 'custom', or 'onesample'
                            % 'group': use DAT.BETWEENPERSON.group or 
                            % DAT.BETWEENPERSON.contrasts{c}.group;
                            % @lukasvo76: compare groups without controlling for covariates
                            % 'custom': use all columns of table object 
                            % DAT.BETWEENPERSON.contrasts{c};
                            % @lukasvo76: covariates to control for in addition to a group factor
                            % 'onesample': use intercept only
                            % @lukasvo76: no group factor, no covariates -
                            % one sample t-test with robust option
                            % (contrary to c_univariate_contrast_maps.m)
dorobfit_parcelwise = false;  % true runs robust parcelwise regression (CANlab's robfit_parcelwise() function) rather than voxel-based GLM (CANlab's regress() function) % added by @lukasvo76 May 2022
    % robfit_parcelwise options
    csf_wm_covs = false; % true adds global wm & csf regressors at second level
    remove_outliers = false; % true removes outlier images/subjects based on mahalanobis distance 

    
%% C2A_SECOND_LEVEL_REGRESSION
%-----------------------------

% NOTE: the first option also applies to c_univariate_contrast_maps_ scripts, but the
% use of c2a_second_level_regression is preferred since it has more
% flexible options for scaling, design specification, and regression method

maskname_glm = which('gray_matter_mask_sparse.img'); %lukasvo76 edited: default use of sparse gray matter mask; maskdir now defined in a_set_up_paths_always_run_first script; if you do not want to mask, change to []; if you want to use a custom mask, put it in maskdir and change name here.
save_figures = false; % true saves .svg files of all figures generated by c2a_second_level_regression.m (slow, takes up space) % added by @lukasvo76 May 2022
q_threshold = .05; % threshold for FDR-corrected display items
p_threshold = .005; % threshold for uncorrected display items
k_threshold = 10; % extent threshold for both corrected and uncorrected display items


%% prep_3b_run_SVMs_on_contrasts_and_save options 
% --------------------------------------------------------------------
dosubjectnorm = false;      % default false     normalize_each_subject_by_l2norm; can help with numerical scaling and inter-subject scaling diffs
dozscoreimages = false;     % default false     Z-score each input image, removing image mean and forcing std to 1. Removes overall effects of image intensity and scale. Can be useful across studies but also removes information. Use judiciously.
dosavesvmstats = true;      % default true      Save statistics and weight map objects for SVM contrasts
dobootstrap = false;        % default false     Takes a lot of time, hence only use true for final analysis, since this takes a lot of time, especially if boot_n is set to the default 10k samples
boot_n = 10000;             % default number of bootstrap samples       Reduce number for quick results
parallelstr = 'parallel';   % parallel proc for boot.   'parallel' or 'noparallel'


%% prep_3c_run_SVMs_on_contrasts_masked options 
% --------------------------------------------------------------------
% see prep_3b options above as well as the following:
maskname_svm = which('gray_matter_mask_sparse.img');


%% prep_3d_run_SVMs_betweenperson_contrasts options
%---------------------------------------------------------------------
% see prep_3b & prep_3c options above as well as the following:
myscaling_svm_between = 'raw'; % see above


%% prep_4_apply_signatures_and_save options 
% --------------------------------------------------------------------
use_scaled_images = false; % @lukasvo76: change to true to use z-scored images - see above


%% emosymp_m1_s5_predict_symptom_ratings_lasso_pcr options
% --------------------------------------------------------------------
% % see prep_3b options above as well as the following:
% maskname_pcr = which('gray_matter_mask_sparse.img'); % see above
% myscaling_pcr = 'raw'; % options are 'raw' or 'scaled'
% dosavepcrstats = true; % see above
% % lukasvo76: this refers to a study-specific script for the emosymp study,
% % need to work on a more generic version


%% LaBGASCORE_SECONDLEVEL_CREATE_SINGLE_TRIAL_FMRI_DATA_ST_OBJ & LaBGASCORE_SECONDLEVEL_RUN_MVPA_REGRESSION_SINGLE_TRIAL
%-----------------------------------------------------------------------------------------------------------------------
zscore_outcome = true; % zscores behavioral outcome variable (fmri_dat.Y) prior to fitting models
maskname_mvpa_reg_st = which('gray_matter_mask_sparse.img'); % see above
myscaling_mvpa_reg_st = 'raw'; % options are 'raw', 'centerimages', 'zscoreimages', 'l2norm_images'
dosavemvparegstats = true; % see above


%% z_batch_publish_everything, z_batch_publish_analyses options 
% --------------------------------------------------------------------
% Enter string for which analyses to run, in any order
%
% 'contrasts'     : Coverage and univariate condition and contrast maps
% 'signatures'    : Pre-defined brain 'signature' responses from CANlab
% 'svm'           : Cross-validated Support Vector Machine analyses for each contrast
% 'bucknerlab'    : Decomposition of each condition and contrast into loadings on resting-state network maps
% 'meta_analysis' : Tests of "pattern of interest" analyses and ROIs derived from CANlab meta-analyses
%
% Default if you run z_batch_publish_analyses is to run all, in this order.
% Or run a custom set:
% batch_analyses_to_run = {'contrasts' 'signatures' 'svm' 'bucknerlab' 'meta_analysis'};
% z_batch_publish_analyses({'svm' 'bucknerlab'})